NONE = "None"
NOR = "Nor"
ANY = "Any"
ADD = "Add"
REMOVE = "Remove"
AND = "And..."
REMOVE_OTHERS = "RemoveOthers"
SETCOLORED = "SetTransparent"
SETOPAQUE = "SetOpaque"
SETAC = "SetAutoColored"
ADDSETCOL = "AddSetColored"
ADDSETTRANS = "AddSetTransparent"
ACTION = "ACTION"
TYPE = "ACTION.TYPE"
R = "ACTION.R"
G = "ACTION.G"
B = "ACTION.B"
